unit DS1307;

{
Copyright (C) 2016 - Ronald Daleske


some procedures in this unit have taken some ideas of the C sources of:

wiringPi from Gordon Henderson
ds1302.c in
https://github.com/nebrius/wiringPi


Licence
=======

 LGPLv2.1 with static linking exception (See COPYING.modifiedLGPL.txt)

}








//    ds1302.c







{$mode objfpc}{$H+}

interface

uses
  Platform,
  RGPIO;



const
  ADR_DS1307_WR : Byte = %11010000;   // 68H -> D0H
  ADR_DS1307_RD : Byte = %11010001;   // 68H -> D1H



var
  DS1307_Data_Array : Array [0..6] of Byte;




{

GPIO2 -> SDA1 / I2C-Daten
GPIO3 -> SCL1 / I2C-Clock

}






procedure SDA_Set;
procedure SCL_Set;
procedure SDA_Clear;
procedure SCL_Clear;
procedure ds1307_Data_Read_Array;
procedure ds1307_Data_Write_Array;



implementation






procedure SDA_Set;
begin
  GPIO2_Set;
end;


procedure SCL_Set;
begin
  GPIO3_Set;
end;


procedure SDA_Clear;
begin
  GPIO2_Clear;
end;


procedure SCL_Clear;
begin
  GPIO3_Clear;
end;


// Read one Byte from Slave, MSB fist, send NACK
function ds_Shift_In_NACK : Byte;
var
  ds_data, ds_couter : Byte;
begin

  // Data as Input
  GPIO_IN_Init(GPIO2);
  MicrosecondDelay(5);

  ds_data:=0;

  for ds_couter:=0 to 7 do begin

    ds_data:=ds_data shl 1;

    if GPIO2_Read then
      ds_data:=ds_data or %00000001;

    SCL_Set;
    MicrosecondDelay(5);

    SCL_Clear;
    MicrosecondDelay(5);

  end; { for ds_couter:=0 to 7 do }

  // Data as Output
  GPIO_OUT_Init(GPIO2);
  SDA_Set;
  MicrosecondDelay(5);

  SCL_Set;
  MicrosecondDelay(5);

  SCL_Clear;
  MicrosecondDelay(5);

  // Data as Output
  SDA_Clear;

  Result:=ds_data;

end; { function ds_Shift_In_NACK }




// Read one Byte from Slave, MSB fist, send ACK
function ds_Shift_In_ACK : Byte;
var
  ds_data, ds_couter : Byte;
begin

  // Data as Input
  GPIO_IN_Init(GPIO2);
  MicrosecondDelay(5);

  ds_data:=0;

  for ds_couter:=0 to 7 do begin

    ds_data:=ds_data shl 1;

    if GPIO2_Read then
      ds_data:=ds_data or %00000001;

    SCL_Set;
    MicrosecondDelay(5);

    SCL_Clear;
    MicrosecondDelay(5);

  end; { for ds_couter:=0 to 7 do }

  // Data as Output
  SDA_Clear;
  GPIO_OUT_Init(GPIO2);
  SDA_Clear;
  MicrosecondDelay(5);

  SCL_Set;
  MicrosecondDelay(5);

  SCL_Clear;
  MicrosecondDelay(5);

  Result:=ds_data;

end; { function ds_Shift_In_NACK }




function ds_Shift_Out(ds_data : Byte) : Boolean;
var
  ds_couter : Byte;
  ds_result : Boolean;
begin

  // Data as Output
  SDA_Clear;
  GPIO_OUT_Init(GPIO2);
  SDA_Clear;
  MicrosecondDelay(5);

  for ds_couter:=0 to 7 do begin

    if (ds_data and %10000000) = 0 then
      SDA_Clear
    else
      SDA_Set;

    ds_data:=ds_data shl 1;

    MicrosecondDelay(5);

    SCL_Set;
    MicrosecondDelay(5);

    SCL_Clear;
    MicrosecondDelay(5);

  end; { for ds_couter:=0 to 7 do }

  // Data as Input
  GPIO_IN_Init(GPIO2);

  MicrosecondDelay(5);

  SCL_Set;
  MicrosecondDelay(5);

  if GPIO2_Read=false then
    ds_result:=true
  else
    ds_result:=false;

  SCL_Clear;
  MicrosecondDelay(5);

  // Data as Output
  SDA_Clear;
  GPIO_OUT_Init(GPIO2);
  SDA_Clear;
  MicrosecondDelay(5);

  Result:=ds_result;

end; { procedure ds_Shift_Out }




procedure ds1307_Data_Write_Word_Address;
begin

  // Start-Sequenz
  SDA_Clear;
  MicrosecondDelay(5);
  SCL_Clear;
  MicrosecondDelay(5);


  ds_Shift_Out(ADR_DS1307_WR);

  ds_Shift_Out(0);


  // Stop-Sequenz
  SCL_Set;
  MicrosecondDelay(5);

  SDA_Set;
  MicrosecondDelay(5);

end; { procedure ds1307_Data_Write_Word_Address }





procedure ds1307_Data_Write_Array;
var
  ds_counter : Byte;
begin


  ds1307_Data_Write_Word_Address;


  // Start-Sequenz
  SDA_Clear;
  MicrosecondDelay(5);
  SCL_Clear;
  MicrosecondDelay(5);

  ds_Shift_Out(ADR_DS1307_WR);

  ds_Shift_Out(0);  // Word -> RTC Table start Position = 0

  for ds_counter:=0 to 6 do begin
    ds_Shift_Out(DS1307_Data_Array[ds_counter]);
  end;


  // Stop-Sequenz
  SCL_Set;
  MicrosecondDelay(5);

  SDA_Set;
  MicrosecondDelay(5);

end; { procedure ds1307_Data_Write_Array }







procedure ds1307_Data_Read_Array;
var
  ds_counter : Byte;
begin


  ds1307_Data_Write_Word_Address;


  // Start-Sequenz
  SDA_Clear;
  MicrosecondDelay(5);
  SCL_Clear;
  MicrosecondDelay(5);

  ds_Shift_Out(ADR_DS1307_RD);

  for ds_counter:=0 to 5 do
    DS1307_Data_Array[ds_counter]:=ds_Shift_In_ACK;

  DS1307_Data_Array[6]:=ds_Shift_In_NACK;


  // Stop-Sequenz
  SCL_Set;
  MicrosecondDelay(5);

  SDA_Set;
  MicrosecondDelay(5);

end; { procedure ds1307_Data_Read_Array }






















end.

